@@ -1,7 +1,7 @@
 /* This testcase is part of GDB, the GNU debugger.
 
-   Copyright 1993, 1994, 1995, 1998, 1999, 2000, 2001, 2004, 2007, 2008, 2009,
-   2010, 2011 Free Software Foundation, Inc.
+   Copyright 1993-1995, 1998-2001, 2004, 2007-2012 Free Software
+   Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -646,10 +646,6 @@ struct struct_with_fnptr *function_struct_ptr = &function_struct;
 
 int main ()
 {
-#ifdef usestubs
-  set_debug_traps();
-  breakpoint();
-#endif
   malloc(1);
   t_double_values(double_val1, double_val2);
   t_structs_c(struct_val1);
